library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity configuration_registers is
  port (
    -- CPU-side interface
    -- This is a strobe
    clk        : in    std_logic;
    rst        : in    std_logic;
    -- active low chip select (tristates data bus when 0).
    sel_n      : in    std_logic;
    -- active low R/W
    r_n        : in    std_logic;
    w_n        : in    std_logic;
    int        : out   std_logic;
    -- 4-bit-wide address bus for configuration registers.
    addr       : in    std_logic_vector (3 downto 0);
    -- data in/out for configuration registers
    data       : inout std_logic_vector (7 downto 0);

    -- all outputs are registered here.
    -- all inputs are registered elsewhere.
    --
    -- error flags, from master/slave processes
    errors     : in    std_logic_vector (6 downto 0);

    -- outgoing, to clear error flags
    errors_clr : out   std_logic;

    -- Only settable internally.
    statuses   : in    std_logic_vector (2 downto 0);

    -- only settable by user.
    configs    : out   std_logic_vector (1 downto 0);

    -- Only settable by user.
    transmit   : out   std_logic_vector (7 downto 0);

    -- Only settable internally.
    receive    : in    std_logic_vector (7 downto 0);
    set_rdre   : out   std_logic;

    -- Only settable by user.
    id_mask    : out   std_logic_vector (5 downto 0);

    -- Only settable by user.
    id_filter  : out   std_logic_vector (5 downto 0);

    -- Notify the core state machine that a new byte is waiting in the
    -- transmit register.
    reset_tdre : out   std_logic;

    clkdiv1    : out   std_logic_vector (3 downto 0);
    clkdiv2    : out   std_logic_vector (7 downto 0)

    );
end configuration_registers;

architecture Behavioral of configuration_registers is
  -- Not registered 
  signal s_idle, s_tdre, s_rdre, s_error : std_logic;

  -- shortframe - A frame received was shorter than expected
  -- noslaveresp - one or more slaves is failing to respond within
  --   the allotted period
  -- xmit - the transmitted data is not the same as the received data.
  -- framing - a short or long frame was detected (master, slave)
  -- parity - a parity error occurred in the identifier (slave)
  -- cksum - checksum error in the packet (master)
  -- overrun - a buffer overrun has occurred
  -- Not registered 
  signal e_shortframe, e_noslaveresp, e_xmit, e_framing, e_parity,
         e_cksum, e_overrun              : std_logic;

  -- Interrupt masks (all r/w)
  -- Registered
  signal i_receive, i_transmit, i_error  : std_logic;
  -- Configuration items (all read/write)
  -- Registered
  signal cfg_id_mask, cfg_id_filter      : std_logic_vector(5 downto 0);
  -- Registered
  signal cfg_clkdiv1                     : std_logic_vector(3 downto 0);
  signal cfg_clkdiv2                     : std_logic_vector(7 downto 0);
  -- Registered
  signal cfg_masterslave, cfg_autobaud   : std_logic;
  -- Transmit buffer: read/write.  Receive buffer: read only
  signal transmit_reg                    : std_logic_vector (7 downto 0);
  signal reading, writing                : std_logic;
  signal data_reg                        : std_logic_vector (7 downto 0);
  signal reading_last, writing_last      : std_logic;
begin
  process (clk, rst)
  begin
    
    if (rst = '1') then
      -- receive - RIE, receive interrupt enable.  Interrupt when a
      --   byte is received.
      -- transmit - TIE, transmit interrupt enable.  Interrupt when
      --   transmit data register is empty.
      -- error - error interrupt enable.  Interrupt when an error condition
      --   exists.
      i_receive <= '1'; i_transmit <= '0'; i_error <= '1';

      cfg_id_mask  <= "000000";
      cfg_id_filter <= "000000";
      -- slow!
--      cfg_clkdiv1       <= "1111";
--      cfg_clkdiv2       <= "11111111";
      -- 300 baud
--      cfg_clkdiv1       <= "0001";
--      cfg_clkdiv2       <= "10000000";
      -- 1200 baud
      --cfg_clkdiv1       <= "0000";
      --cfg_clkdiv2       <= "01100000";

      -- original
      cfg_clkdiv1       <= "0000";
      cfg_clkdiv2       <= "00000010";
      -- 1 = slave, 0 = master.  Default master.
      cfg_masterslave <= '0';
      cfg_autobaud    <= '0';

      transmit_reg <= "00000000";

      errors_clr <= '0';
      data_reg <= "00000000";
      reset_tdre <= '0';
      set_rdre <= '0';

    elsif (rising_edge(clk)) then
      writing_last <= writing;
      reading_last <= reading;
      -- Set defaults
      errors_clr <= '0';
      reset_tdre <= '0';
      set_rdre <= '0';
       
      -- latch the internal registers on the *rising* edge of "writing", or
      -- the first clock where both w_n and sel_n are '1'.
      if (writing = '0' and writing_last = '1') then
        -- configuration/control registers
        case addr is
          -- 0000 - Control register 1
          when "0000" =>
            cfg_autobaud    <= data(1);
            cfg_masterslave <= data(0);
            i_receive <= data(7);
            i_transmit <= data(6);
            i_error <= data(5);
          when "0100" =>
            transmit_reg <= data;
            -- Mark the transmit data register not empty (reset to '0')
            reset_tdre <= '1';
          when "0110" => cfg_id_mask <= data (5 downto 0);
          when "0111" => cfg_id_filter <= data (5 downto 0);
          when "1000" => cfg_clkdiv1 <= data (3 downto 0);
          when "1001" => cfg_clkdiv2 <= data;
          when others => null;
        end case;
      -- latch the read register on the *falling* edge of "reading", or
      -- the first clock where both r_n and sel_n are '0'.
      elsif (reading = '1' and reading_last = '0') then
        -- else drive the correct signal
        case addr is
          -- 0000 - Control register1 
          when "0000" => data_reg <= i_receive & i_transmit & i_error & "000" &
                                     cfg_autobaud & cfg_masterslave;
          -- 0010 - Status Register 1 (status, read only)
          when "0010" => data_reg <= "0000" &
                                     s_error & s_idle & s_tdre & s_rdre;
          -- 0011 - Status Register 2 (errors)
          when "0011" =>
            data_reg <= "0" & e_shortframe & e_noslaveresp & e_xmit &
                        e_framing & e_parity & e_cksum & e_overrun;
            -- the errors register is a unique case - reading the register
            -- clears some of the error flags.  
            errors_clr <= '1';
          -- 0100 - Transmit Data Register
          when "0100" => data_reg <= transmit_reg;
          -- 0101 - Receive Data Register (read only)
          when "0101" =>
            data_reg <= receive;
            set_rdre <= '1';
          -- 0110 - Slave ID mask
          when "0110" => data_reg <= "00" & cfg_id_mask;
          -- 0111 - Slave ID filter
          when "0111" => data_reg <= "00" & cfg_id_filter;
          -- 1000 - Clkdiv1
          when "1000" => data_reg <= "0000" & cfg_clkdiv1;
          -- 1001 - Clkdiv2
          when "1001" => data_reg <= cfg_clkdiv2;
          -- invalid address selected.
          when others => null;
        end case;        
        
      end if;
    end if;  
  end process;

  -- Tristate the data bus when we're not reading or not selected.
  data <= "ZZZZZZZZ" when (reading = '0') else data_reg;
    
  -- Line driver/output registers
  transmit <= transmit_reg;

  configs <= cfg_masterslave & cfg_autobaud;
  id_mask <= cfg_id_mask;
  id_filter <= cfg_id_filter;

  -- input drivers
  s_idle <= statuses(2);
  s_tdre <= statuses(1);
  s_rdre <= statuses(0);
  -- s_error is a special case - it enables the user to check with one
  -- register read whether an error has been flagged.
  s_error <= (e_shortframe or e_noslaveresp or e_xmit or e_framing or
              e_parity or e_cksum or e_overrun);
  
  e_shortframe <= errors(6);
  e_noslaveresp <= errors(5);
  e_xmit <= errors(4);
  e_framing <= errors(3);
  e_parity <= errors(2);
  e_cksum <= errors(1);
  e_overrun <= errors(0);

  -- convenience signals
  reading <= '1' when (r_n='0' and sel_n='0') else '0';
  -- "writing" should be exclusive of "reading".  Ideally this should never
  -- happen.
  writing <= '1' when (w_n='0' and sel_n='0' and r_n='1') else '0';

  -- Set the external interrupt flags
  int <= '1' when (i_receive = '1' and s_rdre = '0') else
         '1' when (i_transmit = '1' and s_tdre = '1') else
         '1' when (i_error = '1' and
                   (e_shortframe or e_noslaveresp or e_xmit or e_framing or
                    e_parity or e_cksum or e_overrun) = '1') else
         '0';

  clkdiv1 <= cfg_clkdiv1;
  clkdiv2 <= cfg_clkdiv2;
end Behavioral;
