library IEEE;
use IEEE.STD_LOGIC_1164.all;
use IEEE.STD_LOGIC_ARITH.all;
use IEEE.STD_LOGIC_UNSIGNED.all;

-- Uncomment the following lines to use the declarations that are
-- provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity core_state_machine is
  port (
    clk                    : in  std_logic;
    bitclk                 : in  std_logic;
    rst                    : in  std_logic;
    ser_in                 : in  std_logic;
    ser_out                : out std_logic;

    -- statuses
    statuses               : out std_logic_vector (2 downto 0);
    -- only settable by user.
    -- masterslave/autobaud
    configs                : in  std_logic_vector (1 downto 0);
    -- Errors, settable internally, clearable by users
    errors                 : out std_logic_vector (6 downto 0);
    errors_clr             : in  std_logic;

    -- Only settable by user.
    transmit               : in  std_logic_vector (7 downto 0);
    reset_tdre             : in  std_logic;
    -- Only settable internally.
    receive                : out std_logic_vector (7 downto 0);
    set_rdre               : in  std_logic;

    receiver_sr            : in  std_logic_vector (7 downto 0);
    receiver_rst           : out std_logic;
    receiver_done          : in  std_logic;
    receiver_framing_error : in  std_logic;

    transmitter_sr         : out std_logic_vector (7 downto 0);
    transmitter_load       : out std_logic;
    transmitter_done       : in  std_logic;
    transmitter_bypass     : out std_logic;
    transmitter_error      : in  std_logic;

    checksum               : in  std_logic_vector (7 downto 0);
    checksum_load          : out std_logic_vector (7 downto 0);
    checksum_strb          : out std_logic;
    checksum_clr           : out std_logic;

    parity_check           : out std_logic_vector (5 downto 0);

    cfg_id_mask            : in  std_logic_vector (5 downto 0);
    cfg_id_filter          : in  std_logic_vector (5 downto 0);

    PR1, PR0               : in  std_logic
    );
end core_state_machine;

architecture Behavioral of core_state_machine is
  type frame_length_t is (
    Length_2,
    Length_4,
    Length_8
  );

  type slave_rcv_state_t is (
    s_idle,                     -- Nothing pending, waiting for break
    s_break1,                   -- Low phase, waiting for high phase
    s_break2,                   -- high phase of break
    s_sync,                     -- synchronizing
    s_wait_rcv_id,              -- Wait for the start bit of the ID
    s_rcv_id,                   -- Receiving the ID
    s_verify_id,                -- Comparing masks to ID for a match
    s_frame_length,             -- Compute the expected packet length
    s_wait_send_rcv,            -- Wait for a slave response or a local
                                -- slave transmission
    s_wait_rcv_byte,            -- Wait for the next byte
    s_rcv_byte,                 -- start up the receiver
    s_wait_check_checksum,      -- wait one cycle
    s_check_checksum            -- check a received checksum
    );
  type slave_xmit_state_t is (
    s_wait_idle,
    s_idle,                     -- Nothing pending, waiting for break
    s_wait_xmit_byte,           -- Wait for the next byte
    s_latch_xmit_byte,          -- Wait for the transmitter to latch the byte.
    s_xmit_byte,                -- start up the transmitter
    s_xmit_checksum,            -- transmit a checksum
    s_wait_xmit_checksum,       -- Wait for the checksummer
    s_latch_xmit_checksum       -- Wait for the transmitter to latch the byte.
    );
  type master_state_t is (
    s_idle,                     -- Nothing pending, waiting for break
    s_break1,                   -- Initial low phase of break
    s_break2,                   -- High phase of break
    s_latch_sync,               -- Latch the synchronization field.
    s_sync,                     -- sending out synch field
    s_latch_send_id,            -- latch the ID.
    s_send_id                   -- sending out ID
    );

  signal   master_state                         : master_state_t;
  signal   slave_xmit_state                     : slave_xmit_state_t;
  signal   slave_rcv_state                      : slave_rcv_state_t;
  -- this counter is used for all sorts of timing operations
  signal   bit_counter                          : std_logic_vector(7 downto 0);
  signal   bit_counter_rst                      : std_logic;

  signal   header_timeout, frame_timeout        : std_logic;
  -- This counter is used for transmission and reception counting
  -- of bytes sent or received
  signal   byte_counter1                        : std_logic_vector(3 downto 0);
  signal   byte_counter2                        : std_logic_vector(3 downto 0);
  signal   byte_counter_load                    : std_logic;
  signal   byte_counter_strb1, byte_counter_strb2
                                                : std_logic;
  signal   checksum_strb1, checksum_strb2       : std_logic;
 
  signal   break_start                          : std_logic;

  signal   status_idle, status_rdre, status_tdre, status_tdre1,
           status_tdre2                         : std_logic;
  signal   cfg_masterslave, cfg_autobaud        : std_logic;
  signal   e_shortframe, e_shortframe1, e_shortframe2, e_noslaveresp,
           e_xmit, e_xmit1, e_xmit2, e_framing, e_parity,
           e_cksum, e_overrun                   : std_logic;

  constant T_BUS_IDLE                           : integer := 25000;
  -- Synch break slave threshold (bit time).
  constant T_SBRKTS                             : integer := 11;
  -- Synch break delimiter minimum bit time.
  constant T_SYNDEL                             : integer := 1;
  -- Synch break low phase master minimum (bit time).
  constant T_SYNBRK                             : integer := 13;

  constant T_HEADER_MIN                         : integer := 34;
  constant T_HEADER_MAX                         : integer := 49;

  -- Minimum Length of Message frame = 10*N_DATA + 44
  constant T_FRAME_MIN_2                        : integer := 64;
  constant T_FRAME_MIN_4                        : integer := 84;
  constant T_FRAME_MIN_8                        : integer := 124;

  -- Maximum Length of Message frame = (T_FRAME_MIN + 1) * 1.4
  constant T_FRAME_MAX_2                        : integer := 91;
  constant T_FRAME_MAX_4                        : integer := 119;
  constant T_FRAME_MAX_8                        : integer := 175;

  signal   break_counter                        : std_logic_vector(3 downto 0);
  signal   ser_in_last                          : std_logic;
  signal   frame_length                         : frame_length_t;
begin

  bitcounter : process (bitclk, bit_counter_rst)
  begin
    if (bit_counter_rst = '1') then
      bit_counter <= "00000000";
    elsif (rising_edge(bitclk)) then
      bit_counter <= bit_counter + 1;
    end if;
  end process bitcounter;
  -- Set the two timeouts according to the currently selected length of
  -- packet (minus T_SBRKTS, since the breakdetect only outputs a '1' after
  -- T_SBRKTS consecutive zeros.)
  header_timeout <= '1' when (bit_counter >= T_HEADER_MAX) else
                    '0';
  frame_timeout <= '1' when ((bit_counter >= (T_FRAME_MAX_2) and
                              frame_length = Length_2) or
                             (bit_counter >= (T_FRAME_MAX_4) and
                              frame_length = Length_4) or
                             (bit_counter >= (T_FRAME_MAX_8) and
                              frame_length = Length_8)) else
                   '0';
  
  breakdetect : process (bitclk, rst)
  begin
    if (rst = '1') then
      break_counter <= "0000";
      ser_in_last <= '0';
    elsif (rising_edge(bitclk)) then
      ser_in_last <= ser_in;
      if (ser_in = '1') then
        break_counter <= "0000";
      elsif (ser_in = '0' and ser_in_last = '0' and break_start = '0') then
        break_counter <= break_counter + 1;
      end if;
    end if;
  end process breakdetect;
  -- T_SBRKTS-1 because ser_in_last is needed to recognize the break
  break_start <= '1' when break_counter >= (T_SBRKTS-1) else '0';

  bytecounter1 : process (clk, rst)
  begin
    if (rst = '1') then
      byte_counter1 <= "0000";
    elsif (rising_edge(clk)) then
      if (byte_counter_strb1 = '1') then
        byte_counter1 <= byte_counter1 - 1;
      elsif (byte_counter_load = '1') then
        if    (frame_length = Length_2) then
          byte_counter1 <= "0010";
        elsif (frame_length = Length_4) then
          byte_counter1 <= "0100";
        elsif (frame_length = Length_8) then
          byte_counter1 <= "1000";
        else
          byte_counter1 <= "0000";
        end if;
      else
        byte_counter1 <= byte_counter1;
      end if;
    end if;
  end process bytecounter1;

  bytecounter2 : process (clk, rst)
  begin
    if (rst = '1') then
      byte_counter2 <= "0000";
    elsif (rising_edge(clk)) then
      if (byte_counter_strb2 = '1') then
        byte_counter2 <= byte_counter2 - 1;
      elsif (byte_counter_load = '1') then
        if    (frame_length = Length_2) then
          byte_counter2 <= "0010";
        elsif (frame_length = Length_4) then
          byte_counter2 <= "0100";
        elsif (frame_length = Length_8) then
          byte_counter2 <= "1000";
        else
          byte_counter2 <= "0000";
        end if;
      else
        byte_counter2 <= byte_counter2;
      end if;
    end if;
  end process bytecounter2;
  
  master_state_machine : process (clk, rst)
   begin                      
    if (rst = '1') then
      master_state <= s_idle;
      e_xmit1 <= '0';
      status_tdre1 <= '0';
    elsif (rising_edge(clk)) then
      -- defaults:
      if (reset_tdre = '1') then
        status_tdre1 <= '0';
      end if;
      if (errors_clr = '1') then
        e_xmit1 <= '0';
      end if;
      
      case master_state is
        when s_idle  =>
          -- If there's a byte pending, continue.
          if (slave_rcv_state = s_idle and status_tdre = '0' and
              cfg_masterslave='0') then
            master_state <= s_break1;
          else
            master_state <= s_idle;
          end if;
          
        when s_break1 =>
          -- Allow a 2-bit error in the low phase of the break
          if (bit_counter = (T_SYNBRK+2)) then
            -- start high phase of break
            master_state <= s_break2;
          else
            master_state <= s_break1;
          end if;
          
        when s_break2 =>
          -- Allow a 1-bit error in the high phase of the break
          if (bit_counter = (T_SYNBRK+T_SYNDEL+3)) then
            master_state <= s_latch_sync;
          else
            master_state <= s_break2;
          end if;
          
        -- The transmitter is operating at the much slower bitclock.  Just
        -- wait for it to latch the data before moving on.
        when s_latch_sync =>
          if (transmitter_done = '0') then
            -- The transmitter latched the byte.  Move on to the next state.
            master_state <= s_sync;
          else
            -- Keep waiting
            master_state <= s_latch_sync;
          end if;

        when s_sync =>
          if (transmitter_done = '0') then
            master_state <= s_sync;
          else -- (transmitter_done = '1')
            if (transmitter_error='1') then
              e_xmit1 <= '1';
            end if;
            master_state <= s_latch_send_id;
          end if;
          
        -- The transmitter is operating at the much slower bitclock.  Just
        -- wait for it to latch the data before moving on.
        when s_latch_send_id =>
          if (transmitter_done = '0') then
            -- The transmitter latched the byte.  Move on to the next state.
            master_state <= s_send_id;
            status_tdre1 <= '1'; 
          else
            -- Keep waiting
            master_state <= s_latch_send_id;
          end if;

        when s_send_id =>
          if (transmitter_done = '0') then
            master_state <= s_send_id;
          else -- (transmitter_done = '1')
            if (transmitter_error='1') then
              e_xmit1 <= '1';
            end if;
            -- done transmitting the ID.  Now just wait for a slave response.
            master_state <= s_idle;
          end if;

        when others =>
          master_state <= s_idle;
      end case;
    end if;
  end process master_state_machine;

  
  slave_rcv_state_machine  : process (clk, rst)
  begin
    if (rst = '1') then
      slave_rcv_state <= s_idle;
      -- Specify defaults for all registered outputs
      receive <= "00000000";
      -- defaults - by default neither the receive data register nor the
      --            transmit data register are full.
      status_rdre <= '1';

      e_shortframe1 <= '0' ; e_noslaveresp <= '0'; 
      e_framing <= '0'; e_parity <= '0'; e_cksum <= '0'; e_overrun <= '0';

      byte_counter_strb1 <= '0';
      receiver_rst <= '0';
      checksum_strb1 <= '0';
      frame_length <= Length_8;
      receiver_rst <= '1';
      
    elsif (rising_edge(clk)) then
      -- Registered output defaults 
      byte_counter_strb1 <= '0';
      receiver_rst <= '0';
      checksum_strb1 <= '0';
      if (set_rdre = '1') then
        status_rdre <= '1';
      end if;
      if (errors_clr = '1') then
        e_shortframe1 <= '0' ; e_noslaveresp <= '0'; 
        e_framing <= '0'; e_parity <= '0'; e_cksum <= '0'; e_overrun <= '0';
      end if;
      
      
      case slave_rcv_state is
        -- Bus is idle, waiting for break
        when s_idle =>
          -- Default packet length here is 8 
          frame_length <= Length_8;
          if (break_start = '1') then
            -- detect break - reset the bit counter 
            slave_rcv_state <= s_break1;
          else
            slave_rcv_state <= s_idle;
          end if;

        -- Break continued, waiting for synchronization delimiter 
        when s_break1 => 
          if (ser_in = '1') then
            slave_rcv_state <= s_break2;
          elsif (header_timeout = '1') then
            -- Timeout while waiting for the synch delimiter - long start
            -- of break (bus shorted?)
            slave_rcv_state <= s_idle;
          else
            slave_rcv_state <= s_break1;
          end if;
          
        -- Break continued (high phase), waiting for start bit of the synch
        -- field.
        when s_break2 =>
          -- just assume that a '0' marks the end of the synch break field,
          -- without regard to the minimum length of the synch delimiter.
          if (ser_in = '0') then
            -- The bit sampler must be synchronized here.
            -- Also, we need to do any fine calibration of the bit timing, but
            -- that's future work.
            slave_rcv_state <= s_sync;
            -- start up the receiver
            receiver_rst <= '1';
          elsif (header_timeout = '1') then
            slave_rcv_state <= s_idle;
          else
            slave_rcv_state <= s_break2;
          end if;
          
        -- synch field, waiting for 8 bits + stop bit.
        when s_sync =>
          if (receiver_done = '1') then
            -- Be conservative here.  A framing error in the synch field is
            -- probably a bad sign, so don't attempt to continue.  Don't
            -- bother flagging e_framing, either, since there will be no byte
            -- in the receive register to associate with it.
            if (receiver_sr="01010101" and receiver_framing_error = '0') then
              slave_rcv_state <= s_wait_rcv_id;
            else
              slave_rcv_state <= s_idle;
            end if;
          else
            -- keep waiting
            slave_rcv_state <= s_sync;
          end if;

       when s_wait_rcv_id =>
          if (ser_in = '0') then
            slave_rcv_state <= s_rcv_id;
            -- start up and synchronize the receiver
            receiver_rst <= '1';
          elsif (header_timeout = '1') then
            slave_rcv_state <= s_idle;
          else
            slave_rcv_state <= s_wait_rcv_id;
          end if;
           
        when s_rcv_id =>
          if (receiver_done = '1') then
            -- Next state is to check the parity of the identifier received
            slave_rcv_state <= s_verify_id;
          else -- (receiver_done = '0') then
            slave_rcv_state <= s_rcv_id;
          end if;
          
        when s_verify_id =>
          -- check the parity and the mask
          if (receiver_sr(7 downto 6) = PR1 & PR0) then
            if ((cfg_id_mask and receiver_sr(5 downto 0)) = cfg_id_filter) then
              -- Parity and mask indicate the message is one we should pay
              -- attention to.
              -- Only flag a framing error if the ID is for us.
              if (receiver_framing_error='1') then
                e_framing <= '1';
              end if;
              -- Check for an overrun error
              if (status_rdre = '0') then
                e_overrun <= '1';
              end if;
              -- Flag data ready
              status_rdre <= '0';
              -- Latch the data
              receive <= receiver_sr;
              slave_rcv_state <= s_frame_length;
            else
              -- The data's not for us
              slave_rcv_state <= s_idle;
            end if;
          else
            -- The parity bits are corrupted.
            -- ignore it, and wait for the next break.
            e_parity <= '1';
            slave_rcv_state <= s_idle;
          end if;
          
        when s_frame_length =>
          if (receiver_sr(5 downto 1) = "11111") then
            -- Identifiers 3E and 3F are for future use.  Proper behavior
            -- is not well defined in the spec, so we'll just punt here
            -- for now.
            slave_rcv_state <= s_idle;
          else
            -- Initialize the byte counter with the appropriate number of bytes
            -- based on the identifier so we know when the appropriate
            -- timeout has been reached.
            if    (receiver_sr(5) = '0') then
              frame_length <= Length_2;
            elsif (receiver_sr(5 downto 4) = "10") then
              frame_length <= Length_4;
            -- Implied:  else frame_length <= Length_8; 
            end if;

            slave_rcv_state <= s_wait_send_rcv;
          end if;
          
        when s_wait_send_rcv =>
          if (ser_in = '0') then
            slave_rcv_state <= s_wait_rcv_byte;
          elsif (frame_timeout = '1') then
            -- No slave response or other activity within the timeout period.
            e_shortframe1 <= '1';
            e_noslaveresp <= '1';
            slave_rcv_state <= s_idle;
          else
            slave_rcv_state <= s_wait_send_rcv;
          end if;

        when s_wait_rcv_byte =>
          if (ser_in = '0') then
            slave_rcv_state <= s_rcv_byte;
            -- start up and synchronize the receiver
            receiver_rst <= '1';
          elsif (frame_timeout = '1') then
            e_shortframe1 <= '1';
            slave_rcv_state <= s_idle;
          else
            slave_rcv_state <= s_wait_rcv_byte;
          end if;

        when s_rcv_byte =>
          if (break_start = '1') then
            -- Problem - someone else has initiated a break.
            slave_rcv_state <= s_break1;
          elsif (receiver_done = '1') then
            -- strobe the byte counter (decrement)
            byte_counter_strb1 <= '1';
            -- strobe the checksummer
            if (slave_xmit_state = s_idle) then
              checksum_strb1 <= '1';
            end if;

            -- If this is the checksum byte, delay setting error flags
            -- until we've latched the checksum into the RDR, so all
            -- error flags will get set at the same time.
            if (byte_counter1 = 0) then
              slave_rcv_state <= s_wait_check_checksum;
            else
              -- The byte has finished being received.
              if (receiver_framing_error='1') then
                e_framing <= '1';
              end if;
              -- Check for an overrun error
              if (status_rdre = '0') then
                e_overrun <= '1';
              end if;
              -- Flag data ready
              status_rdre <= '0';
              -- Latch the data
              receive <= receiver_sr;

              slave_rcv_state <= s_wait_rcv_byte;
            end if;
          else -- (receiver_done = '0') then
            slave_rcv_state <= s_rcv_byte;
          end if;
          
        when s_wait_check_checksum =>
          -- the checksummer takes an additional cycle to return
          -- the checksum.  Wait one more cycle.
          slave_rcv_state <= s_check_checksum;

        when s_check_checksum =>
          -- Check for errors with the received checksum
          if (receiver_framing_error='1') then
            e_framing <= '1';
          end if;
          -- Check for an overrun error
          if (status_rdre = '0') then
            e_overrun <= '1';
          end if;
          if (checksum /= receiver_sr) then
            -- flag a bad checksum
            e_cksum <= '1';
          end if;
          -- Flag data ready
          status_rdre <= '0';
          -- Latch the checksum
          receive <= receiver_sr;
          slave_rcv_state <= s_idle;
          
          -- invalid state - punt!
        when others =>
          slave_rcv_state <= s_idle;
      end case;
    end if;
  end process slave_rcv_state_machine;

  slave_xmit_state_machine  : process (clk, rst)
  begin
    if (rst = '1') then
      slave_xmit_state <= s_idle;
      -- Specify defaults for all registered outputs
      -- defaults - by default neither the receive data register nor the
      --            transmit data register are full.
      status_tdre2 <= '1';

      e_shortframe2 <= '0' ; e_xmit2 <= '0';

      byte_counter_strb2 <= '0';
      checksum_strb2 <= '0';
      
    elsif (rising_edge(clk)) then
      -- Registered output defaults 
      byte_counter_strb2 <= '0';
      checksum_strb2 <= '0';
      if (reset_tdre = '1') then
        status_tdre2 <= '0';
      end if;
      if (errors_clr = '1') then
        e_shortframe2 <= '0' ; e_xmit2 <= '0';
      end if;
      
      case slave_xmit_state is
        -- Bus is idle, waiting for break
        when s_wait_idle =>
          if (slave_rcv_state = s_idle) then
            slave_xmit_state <= s_idle;
          end if;

        when s_idle =>
          if (slave_rcv_state = s_wait_send_rcv and
              status_tdre = '0' and ser_in /= '0') then
            slave_xmit_state <= s_wait_xmit_byte;
          end if;

        when s_wait_xmit_byte =>
          if (status_tdre = '0' and transmitter_done='1') then
            -- start up the transmitter and latch the data.
            -- strobe the byte counter (decrement)
            byte_counter_strb2 <= '1';
            -- strobe the checksummer
            checksum_strb2 <= '1';
            -- Wait while the byte is latched
            slave_xmit_state <= s_latch_xmit_byte;
          elsif (break_start = '1') then
            -- Problem - someone else has initiated a break.
            slave_xmit_state <= s_wait_idle;
          elsif (frame_timeout = '1') then
            e_shortframe2 <= '1';
            slave_xmit_state <= s_wait_idle;
          else
            slave_xmit_state <= s_wait_xmit_byte;
          end if;

        -- The transmitter is operating at the much slower bitclock.  Just
        -- wait for it to latch the data before moving on.
        when s_latch_xmit_byte =>
          if (transmitter_done = '0') then
            -- The transmitter latched the byte.  Move on to the next state.
            slave_xmit_state <= s_xmit_byte;
            -- Mark the transmit data register empty
            status_tdre2 <= '1'; 
          else
            -- Keep waiting
            slave_xmit_state <= s_latch_xmit_byte;
          end if;
          
        when s_xmit_byte =>
          if (transmitter_done = '0') then
            slave_xmit_state <= s_xmit_byte;
          else -- (transmitter_done = '1')
            if (transmitter_error='1') then
              e_xmit2 <= '1';
            end if;
            if (byte_counter2 = 0) then
              -- Strobe the final "00" so the checksummer can add its
              -- final carry.
              checksum_strb2 <= '1';
              slave_xmit_state <= s_wait_xmit_checksum;
            else
              slave_xmit_state <= s_wait_xmit_byte;
            end if;
          end if;
          
        when s_wait_xmit_checksum =>
          if (frame_timeout = '1') then
            e_shortframe2 <= '1';
            slave_xmit_state <= s_wait_idle;
          else
            -- the checksummer takes an additional cycle to return
            -- the checksum.  Wait one more cycle.
            slave_xmit_state <= s_latch_xmit_checksum;
          end if;

        -- The transmitter is operating at the much slower bitclock.  Just
        -- wait for it to latch the data before moving on.
        when s_latch_xmit_checksum =>
          if (transmitter_done = '0') then
            -- The transmitter latched the byte.  Move on to the next state.
            slave_xmit_state <= s_xmit_checksum;
          else
            -- Keep waiting
            slave_xmit_state <= s_latch_xmit_checksum;
          end if;
          
        when s_xmit_checksum =>
          if (transmitter_done = '0') then
            slave_xmit_state <= s_xmit_checksum;
          else -- (transmitter_done = '1')
            if (transmitter_error='1') then
              e_xmit2 <= '1';
            end if;
            slave_xmit_state <= s_wait_idle;
          end if;
          
          -- invalid state - punt!
        when others =>
          slave_xmit_state <= s_wait_idle;
      end case;
    end if;
  end process slave_xmit_state_machine;
  
  -- State-dependent outputs 
  transmitter_bypass <= '1' when (master_state = s_break1 or
                                  master_state = s_break2) else
                        '0';
  ser_out <= '0' when (master_state = s_break1) else '1';
  byte_counter_load <= '1' when (slave_rcv_state = s_wait_send_rcv and
                                 slave_xmit_state = s_idle) else
                       '0';
  transmitter_load <= '1' when (master_state = s_latch_sync or
                                master_state = s_latch_send_id or
                                slave_xmit_state = s_latch_xmit_byte or
                                slave_xmit_state = s_latch_xmit_checksum) else
                      '0';
  checksum_load <= transmit when (slave_xmit_state = s_latch_xmit_byte) else
                   receiver_sr when (slave_rcv_state = s_wait_rcv_byte and
                                     slave_xmit_state = s_idle) else
                   "00000000";
  transmitter_sr <= checksum when (slave_xmit_state =
                                   s_latch_xmit_checksum) else
                    "01010101" when (master_state = s_latch_sync) else
                    PR1&PR0&transmit(5 downto 0)
                      when (master_state = s_latch_send_id) else
                    transmit;
  status_idle <= '1' when (slave_rcv_state = s_idle and
                           slave_xmit_state = s_idle) else '0';
  checksum_clr <= '1' when (slave_rcv_state = s_idle and
                           slave_xmit_state = s_idle) else '0';
  parity_check <= transmit(5 downto 0) when master_state = s_latch_send_id else
                  receiver_sr(5 downto 0);

  -- any non-state-dependent outputs
  statuses <= status_idle & status_tdre & status_rdre;
  errors <= e_shortframe & e_noslaveresp & e_xmit & e_framing & e_parity &
            e_cksum & e_overrun;

  -- any non-state-dependent inputs
  cfg_masterslave <= configs(1);
  cfg_autobaud <= configs(0);


  -- Glue logic for multiply-driven signals
  status_tdre <= status_tdre1 or status_tdre2;
  e_xmit <= e_xmit1 or e_xmit2;
  e_shortframe <= e_shortframe1 or e_shortframe2;
  checksum_strb <= checksum_strb1 or checksum_strb2;

  -- Start the bit counter running if there is activity on the bus or
  -- the master is not idle or the slave is not idle.
  bit_counter_rst <= '1' when ((master_state = s_idle) and
                               (slave_rcv_state = s_idle
                                and ser_in = '1')) else
                     '0';

end Behavioral;



