library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity lin_controller is
  port (
    clk       : in    std_logic;
    -- strobe for r/w
    rst       : in    std_logic;
    -- enable/select (tristates data bus when 0)
    sel_n     : in    std_logic;
    -- R/W'
    r_n       : in    std_logic;
    w_n       : in    std_logic;
    int       : out   std_logic;
    -- 4-bit-wide address bus for configuration registers.
    addr      : in    std_logic_vector (3 downto 0);
    -- data in/out for configuration registers
    data      : inout std_logic_vector (7 downto 0);
    RxD       : in    std_logic;
    TxD       : out   std_logic;
    clk_out   : out   std_logic;
    rst_out   : out   std_logic
    );
end lin_controller;

architecture structural of lin_controller is
  component core_state_machine
    port (
      clk                    : in  std_logic;
      bitclk                 : in  std_logic;
      rst                    : in  std_logic;
      ser_in                 : in  std_logic;
      ser_out                : out std_logic;
      statuses               : out std_logic_vector (2 downto 0);
      configs                : in  std_logic_vector (1 downto 0);
      errors                 : out std_logic_vector (6 downto 0);
      errors_clr             : in  std_logic;
      transmit               : in  std_logic_vector (7 downto 0);
      reset_tdre             : in  std_logic;
      receive                : out std_logic_vector (7 downto 0);
      set_rdre               : in  std_logic;
      receiver_sr            : in  std_logic_vector (7 downto 0);
      receiver_rst           : out std_logic;
      receiver_done          : in  std_logic;
      receiver_framing_error : in  std_logic;
      transmitter_sr         : out std_logic_vector (7 downto 0);
      transmitter_load       : out std_logic;
      transmitter_done       : in  std_logic;
      transmitter_bypass     : out std_logic;
      transmitter_error      : in  std_logic;
      checksum               : in  std_logic_vector (7 downto 0);
      checksum_load          : out std_logic_vector (7 downto 0);
      checksum_strb          : out std_logic;
      checksum_clr           : out std_logic;
      parity_check           : out std_logic_vector (5 downto 0);
      cfg_id_mask            : in  std_logic_vector (5 downto 0);
      cfg_id_filter          : in  std_logic_vector (5 downto 0);
      PR1, PR0               : in  std_logic
      );
  end component;

  component majority_sampler
    port (
      clk  : in  std_logic;
      rst  : in  std_logic;
      data : in  std_logic;
      dout : out std_logic
      );
  end component;

  component divider is
    port (
      clk, rst   : in  std_logic;
      bitclk     : out std_logic;
      bitclk_x16 : out std_logic;
      clkdiv1    : in  std_logic_vector (3 downto 0);
      clkdiv2    : in  std_logic_vector (7 downto 0)
    );
  end component;

  component parity_generator is
    port (
      data     : in  std_logic_vector (5 downto 0);
      PR1, PR0 : out std_logic
    );
  end component;

  component receiver is
    port (
      bitclk_x16    : in  std_logic;
      rst           : in  std_logic;
      ser_in        : in  std_logic;
      done          : out std_logic;
      par_out       : out std_logic_vector (7 downto 0);
      framing_error : out std_logic
    );
  end component;
  
  component transmitter is
    port (
      bitclk       : in  std_logic;
      rst          : in  std_logic;
      bypass_input : in  std_logic;
      ser_in       : in  std_logic;
      ser_out      : out std_logic;
      done         : out std_logic;
      bypass       : in  std_logic;
      -- parallel input
      par_in       : in  std_logic_vector (7 downto 0);
      par_load     : in  std_logic;
      xmit_error   : out std_logic
    );
  end component;

  component checksum_generator is
    port (
      clk   : in  std_logic;
      clr   : in  std_logic;
      rst   : in  std_logic;
      data  : in  std_logic_vector(7 downto 0);
      strb  : in  std_logic;
      cksum : out std_logic_vector(7 downto 0)
    );
  end component;

  component configuration_registers is
    port (
      clk        : in    std_logic;
      rst        : in    std_logic;
      sel_n      : in    std_logic;
      r_n        : in    std_logic;
      w_n        : in    std_logic;
      int        : out   std_logic;
      addr       : in    std_logic_vector (3 downto 0);
      data       : inout std_logic_vector (7 downto 0);
      errors     : in    std_logic_vector (6 downto 0);
      errors_clr : out   std_logic;
      statuses   : in    std_logic_vector (2 downto 0);
      configs    : out   std_logic_vector (1 downto 0);
      transmit   : out   std_logic_vector (7 downto 0);
      receive    : in    std_logic_vector (7 downto 0);
      set_rdre   : out   std_logic;
      id_mask    : out   std_logic_vector (5 downto 0);
      id_filter  : out   std_logic_vector (5 downto 0);
      reset_tdre : out   std_logic;
      clkdiv1    : out   std_logic_vector (3 downto 0);
      clkdiv2    : out   std_logic_vector (7 downto 0)
--      int_rdrf   : out   std_logic;
--      int_tdre   : out   std_logic;
--      int_err    : out   std_logic
    );
  end component;

  signal configs : std_logic_vector (1 downto 0);
  signal statuses : std_logic_vector(2 downto 0);
  signal parity_check, cfg_id_mask, 
         cfg_id_filter :  std_logic_vector(5 downto 0);
  signal errors : std_logic_vector(6 downto 0);
  signal transmit, receive, receiver_sr, transmitter_sr,
         checksum, checksum_load, clkdiv2
         : std_logic_vector(7 downto 0);
  signal clkdiv1 : std_logic_vector (3 downto 0);
  
  signal receiver_done, receiver_framing_error, receiver_rst,
         transmitter_load, transmitter_done, bitclk, checksum_strb,
         checksum_clr, ser_in, transmitter_bypass, PR0, PR1,
         transmitter_bypass_input, bitclk_x16, errors_clr,
         reset_tdre, transmitter_error, set_rdre : std_logic;
  
begin
  state_machine1: core_state_machine port map (
    clk, bitclk, rst, ser_in, transmitter_bypass_input, statuses, configs,
    errors, errors_clr, transmit, reset_tdre, receive, set_rdre, receiver_sr,
    receiver_rst, receiver_done, receiver_framing_error, transmitter_sr,
    transmitter_load, transmitter_done, transmitter_bypass, transmitter_error,
    checksum, checksum_load, checksum_strb, checksum_clr,
    parity_check, cfg_id_mask, cfg_id_filter, PR1, PR0
  );
  sampler1: majority_sampler port map (bitclk_x16, rst, RxD, ser_in);
  divider1: divider port map (clk, rst, bitclk, 
                              bitclk_x16, clkdiv1, clkdiv2);
  paritygen1: parity_generator port map (parity_check, PR1, PR0);
  receiver1: receiver port map (bitclk_x16, receiver_rst, ser_in,
                                receiver_done, receiver_sr,
                                receiver_framing_error);
  transmitter1: transmitter port map (bitclk, rst, 
                                      transmitter_bypass_input, ser_in, TxD,
                                      transmitter_done, transmitter_bypass,
                                      transmitter_sr, transmitter_load,
                                      transmitter_error);
  cksum1: checksum_generator port map (clk, checksum_clr, rst, checksum_load,
                                       checksum_strb, checksum);
  confreg1: configuration_registers port map (clk, rst, sel_n, r_n, w_n, int,
                                              addr, data, errors, errors_clr,
                                              statuses, configs, 
                                              transmit, receive, set_rdre,
                                              cfg_id_mask, cfg_id_filter,
                                              reset_tdre, clkdiv1, clkdiv2);


  clk_out <= clk;
  rst_out <= rst;

end structural;
